VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "CWayPointX"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit
Option Compare Text
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' Class CWayPointX
' Author: Chip Pearson, Pearson Sofware Consulting, LLC, www.cpearson.com
' Date: 23-June-2002
'
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' This code is explicitly granted to the Public Domain.
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
'
' Note that only the Name and latitude and longitude properties
' of the MapPoint WayPoint object are preserved in this
' class.
'
' If you want to preserve other properties, do the following:
'   1) Add a private variable for the property. Example:
'           Private pOvernights As Long
'   2) Create a Property Get procedure for that property. Example:
'           Property Get Overnights() As Long
'               Overnights = pOvernights
'           End Property
'   3) Assign the value of the WayPoint to the private variable, in the
'      SetWayPoint procedure. Example:
'           pOvernights = WP.Overnights
'
'   Repeats these steps for each property you want to preserve.
'
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Private pName As String
Private pLat As Double
Private pLong As Double
Private pMapFileName As String

Property Get Name() As String
    Name = pName
End Property
Property Get Latitude() As Double
    Latitude = pLat
End Property
Property Get Longitude() As Double
    Longitude = pLong
End Property
Property Get MapFileName() As String
    MapFileName = pMapFileName
End Property
Property Let MapFileName(FName As String)
    pMapFileName = FName
End Property

Public Sub SetWayPoint(WP As MapPoint.Waypoint)
    pName = WP.Name
    pMapFileName = WP.Location.Parent.FullName
    CalcPos WP.Location.Parent, WP.Location, pLat, pLong
End Sub


''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' The following procedures were written by Gilles Kohl, and can be
' found at www.mp2k.com:
' A More Compact Method for Obtaining Lat/Long : July 18, 2001
' An alternate method for obtaining lat/lon was recently posted to the MapPoint
' newsgroup by Gilles Kohl that works with pushpins and also works in the European version.
'
' Thanks, Gilles, for contributing this code.
'
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' Compute latitude and longitude given a location object
' Author: Gilles Kohl
' (gilles@compuserve.com)
'
' This code is copyrighted freeware - use freely, but please leave this
' header intact. Suggestions and comments welcome.

Private Function Arccos(x As Double) As Double
   If x = 1 Then
      Arccos = 0
      Exit Function
   End If
   Arccos = Atn(-x / Sqr(-x * x + 1)) + 2 * Atn(1)
End Function

Sub CalcPos(objMap As MapPoint.Map, locX As MapPoint.Location, dblLat As Double, dblLon As Double)
   Static locNorthPole As MapPoint.Location
   Static locSantaCruz As MapPoint.Location ' Center of western hemisphere
   Static dblHalfEarth As Double ' Half circumference of the earth (as a sphere)
   Static dblQuarterEarth As Double ' Quarter circumference of the earth (as a sphere)
   Static Pi As Double

   ' Check if initialization already done
   If locNorthPole Is Nothing Then
      Set locNorthPole = objMap.GetLocation(90, 0)
      Set locSantaCruz = objMap.GetLocation(0, -90)

      ' Compute distance between north and south poles == half earth circumference
      dblHalfEarth = objMap.Distance(locNorthPole, objMap.GetLocation(-90, 0))

      ' Quarter of that is the max distance a point may be away from locSantaCruz and still be in western hemisphere
      dblQuarterEarth = dblHalfEarth / 2
      Pi = 3.14159265358979
   End If

   ' Compute latitude from distance to north pole
   dblLat = 90 - 180 * objMap.Distance(locNorthPole, locX) / dblHalfEarth

   Dim l As Double
   Dim d As Double

   ' Compute great circle distance to locX from point on Greenwich meridian and computed Latitude
   d = objMap.Distance(objMap.GetLocation(dblLat, 0), locX)

   ' convert latitude to radian
   l = (dblLat / 180) * Pi

   ' Compute Longitude from great circle distance
   dblLon = 180 * Arccos((Cos((d * 2 * Pi) / (2 * dblHalfEarth)) - Sin(l) * Sin(l)) / (Cos(l) * Cos(l))) / Pi

   ' Correct longitude sign if located in western hemisphere
   If objMap.Distance(locSantaCruz, locX) < dblQuarterEarth Then dblLon = -dblLon
End Sub

''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' End Of Gilles Kohl code block
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

